----------------------------------------------------------------------------------
--  Autor: Victor Ral Arteaga Prez
--
-- Description: Este mdulo se encarga de controlar al ADC mediante el IP XADC, la conversin 
--              tiene una resolucin de 12 bits aunque slo se utilicen 8 para mandarlos al ES32. Slo se 
--              utiliza la parte positiva del puerto diferencial.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity ADC is

PORT(
    CLK       : IN  STD_LOGIC; -- Reloj FGPA a 100MHz.          
    ADC_P     : IN  STD_LOGIC_VECTOR(6 DOWNTO 0); -- Canales diferenciales
    ADC_N     : IN  STD_LOGIC_VECTOR(6 DOWNTO 0); -- de ADC.
    CH        : IN  STD_LOGIC_VECTOR(3 DOWNTO 0); -- Selector de canal de ADC.
    DATA_ADC  : OUT STD_LOGIC_VECTOR(11 DOWNTO 0) -- Valor digital de la conversin
    );
    
end ADC;

architecture Behavioral of ADC is


COMPONENT xadc_wiz_0 is
   port
   (
    daddr_in        : in  STD_LOGIC_VECTOR (6 downto 0);     -- Address bus for the dynamic reconfiguration port
    den_in          : in  STD_LOGIC;                         -- Enable Signal for the dynamic reconfiguration port
    di_in           : in  STD_LOGIC_VECTOR (15 downto 0);    -- Input data bus for the dynamic reconfiguration port
    dwe_in          : in  STD_LOGIC;                         -- Write Enable for the dynamic reconfiguration port
    do_out          : out  STD_LOGIC_VECTOR (15 downto 0);   -- Output data bus for dynamic reconfiguration port
    drdy_out        : out  STD_LOGIC;                        -- Data ready signal for the dynamic reconfiguration port
    dclk_in         : in  STD_LOGIC;                         -- Clock input for the dynamic reconfiguration port
    reset_in        : in  STD_LOGIC;                         -- Reset signal for the System Monitor control logic
    vauxp4          : in  STD_LOGIC;                         -- Auxiliary Channel 4
    vauxn4          : in  STD_LOGIC;
    vauxp5          : in  STD_LOGIC;                         -- Auxiliary Channel 5
    vauxn5          : in  STD_LOGIC;
    vauxp6          : in  STD_LOGIC;                         -- Auxiliary Channel 6
    vauxn6          : in  STD_LOGIC;
    vauxp7          : in  STD_LOGIC;                         -- Auxiliary Channel 7
    vauxn7          : in  STD_LOGIC;
    vauxp12         : in  STD_LOGIC;                         -- Auxiliary Channel 12
    vauxn12         : in  STD_LOGIC;
    vauxp14         : in  STD_LOGIC;                         -- Auxiliary Channel 14
    vauxn14         : in  STD_LOGIC;
    vauxp15         : in  STD_LOGIC;                         -- Auxiliary Channel 15
    vauxn15         : in  STD_LOGIC;
    busy_out        : out  STD_LOGIC;                        -- ADC Busy signal
    channel_out     : out  STD_LOGIC_VECTOR (4 downto 0);    -- Channel Selection Outputs
    eoc_out         : out  STD_LOGIC;                        -- End of Conversion Signal
    eos_out         : out  STD_LOGIC;                        -- End of Sequence Signal
    alarm_out       : out STD_LOGIC;                         -- OR'ed output of all the Alarms
    vp_in           : in  STD_LOGIC;                         -- Dedicated Analog Input Pair
    vn_in           : in  STD_LOGIC
);
end COMPONENT xadc_wiz_0;

-- Seales para el componente "xadc_wiz_0"
signal enable          : std_logic := '0';
signal ready           : std_logic := '0';
signal last_ready      : std_logic := '0';
signal ready_pe_strobe : std_logic := '0';
signal ready_ne_strobe : std_logic := '0';
signal Address_in      : std_logic_vector(7  downto 0);
signal data            : std_logic_vector(15 downto 0);
signal buff_adc        : std_logic_vector(15 downto 0);

begin

U1:  xadc_wiz_0
   port MAP
   (
   daddr_in => Address_in(6 downto 0),
    den_in => enable,
    di_in => (others => '0'),
    dwe_in => '0',
    do_out => data,
    drdy_out => ready,
    dclk_in => clk,
    reset_in => '0',
    vauxp4 => adc_p(1),
    vauxn4 => adc_n(1),
    vauxp5 => adc_p(4),
    vauxn5 => adc_n(4),
    vauxp6 => adc_p(3),
    vauxn6 => adc_n(3),
    vauxp7 => adc_p(0),
    vauxn7 => adc_n(0),
    vauxp12 => adc_p(6),
    vauxn12 => adc_n(6),
    vauxp14 => adc_p(2),
    vauxn14 => adc_n(2),
    vauxp15 => adc_p(5),
    vauxn15 => adc_n(5),
    busy_out => open,
    channel_out => open,
    eoc_out => enable,
    eos_out => open,
    alarm_out => open,
    vp_in => '0',
    vn_in => '0'
);

-- Realiza un Latch de la seal ready para indicar cundo hacer lectura 
-- de la parte positiva del canal diferencial
process (clk)
begin
    if rising_edge(clk) then
        last_ready <= ready;     
    end if;
end process;
ready_pe_strobe <= '1' when (last_ready = '0' and ready = '1') else '0';
ready_ne_strobe <= '1' when (last_ready = '1' and ready = '0') else '0';

-- Cuando "ready_pe_strobe" se pone a '1' mandamos el dato de la conversin
process(clk)
begin
    if rising_edge(clk) then
        if ready_pe_strobe = '1' then
            DATA_ADC <= data(15 DOWNTO 4);
        end if;
    end if;
end process;

-- Proceso que selecciona al canal con el puerto "ch"
process(clk)
begin
    if rising_edge(clk) then
        if ready_ne_strobe = '1' then
            case(ch) is
                when X"0" => Address_in <= X"17";
                when X"1" => Address_in <= X"14";
                when X"2" => Address_in <= X"1E";
                when X"3" => Address_in <= X"16";
                when X"4" => Address_in <= X"15";
                when X"5" => Address_in <= X"1F";
                when X"6" => Address_in <= X"1C";
                when others => Address_in <= X"17";
            end case;
        end if;
    end if;
end process;


end Behavioral;
